@@ -26,22 +26,20 @@ namespace pm {
 
 template <typename Iterator>
 class RandomSubset_iterator : public Iterator {
-   typedef Iterator base_t;
+   using base_t = Iterator;
 public:
-   typedef forward_iterator_tag iterator_category;
-   typedef RandomSubset_iterator<typename iterator_traits<Iterator>::iterator>
-      iterator;
-   typedef RandomSubset_iterator<typename iterator_traits<Iterator>::const_iterator>
-      const_iterator;
+   using iterator_category = forward_iterator_tag;
+   using iterator =RandomSubset_iterator<typename iterator_traits<Iterator>::iterator>;
+   using const_iterator = RandomSubset_iterator<typename iterator_traits<Iterator>::const_iterator>;
 
-   RandomSubset_iterator() {}
+   RandomSubset_iterator() = default;
 
    RandomSubset_iterator(const iterator& it)
       : base_t(static_cast<const typename iterator::base_t&>(it))
       , rg(it.rg)
       , k(it.k) {}
 
-   template <typename SourceIterator, typename suitable=typename suitable_arg_for_iterator<SourceIterator, Iterator>::type>
+   template <typename SourceIterator, typename=typename suitable_arg_for_iterator<SourceIterator, Iterator>::type>
    RandomSubset_iterator(const SourceIterator& cur_arg, int n_arg, int k_arg, const SharedRandomState& random_arg)
       : base_t(prepare_iterator_arg<Iterator>(cur_arg))
       , rg(n_arg, random_arg)
@@ -50,7 +48,7 @@ public:
       toss(typename iterator_traits<Iterator>::iterator_category(), 0);
    }
 
-   template <typename SourceIterator, typename enable=typename std::enable_if<is_const_compatible_with<SourceIterator, Iterator>::value>::type>
+   template <typename SourceIterator, typename=std::enable_if_t<is_const_compatible_with<SourceIterator, Iterator>::value>>
    RandomSubset_iterator(const SourceIterator& end_arg, const SharedRandomState& random_arg)
       : base_t(end_arg)
       , rg(0,random_arg)
@@ -91,12 +89,12 @@ protected:
    template <typename> friend class RandomSubset_iterator;
 private:
    // shadow possible base_t:: operators
-   void operator--();
-   void operator+=(int);
-   void operator-=(int);
-   void operator+(int);
-   void operator-(int);
-   void operator[](int);
+   void operator--() = delete;
+   void operator+=(int) = delete;
+   void operator-=(int) = delete;
+   void operator+(int) = delete;
+   void operator-(int) = delete;
+   void operator[](int) = delete;
 };
 
 template <typename Iterator>
@@ -104,17 +102,30 @@ struct check_iterator_feature<RandomSubset_iterator<Iterator>, end_sensitive> :
 
 template <typename ContainerRef>
 class RandomSubset
-   : public generic_of_subset<RandomSubset<ContainerRef>, typename deref<ContainerRef>::type> {
+   : public generic_of_subset_t<RandomSubset<ContainerRef>, typename deref<ContainerRef>::type> {
 public:
-   typedef typename deref<ContainerRef>::minus_ref base_type;
-   typedef typename container_traits<ContainerRef>::value_type value_type;
-   typedef typename container_traits<ContainerRef>::reference reference;
-   typedef typename container_traits<ContainerRef>::const_reference const_reference;
+   using base_type = typename deref<ContainerRef>::minus_ref;
+   using value_type = typename container_traits<ContainerRef>::value_type;
+   using reference = typename container_traits<ContainerRef>::reference;
+   using const_reference = typename container_traits<ContainerRef>::const_reference;
 
+protected:
+   using alias_t = alias<ContainerRef>;
+   using random_t = UniformlyRandom<long>;
+
+   alias_t base;
+   random_t random_source;
+   long k;
+
+public:
    RandomSubset() {}
 
-   RandomSubset(typename alias<ContainerRef>::arg_type base_arg, long k_arg, const RandomSeed& seed=RandomSeed())
-      : base(base_arg), random_source(seed), k(k_arg)
+   template <typename BaseArg, typename SeedArg,
+             typename=std::enable_if_t<std::is_constructible<alias_t, BaseArg>::value && std::is_constructible<random_t, SeedArg>::value>>
+   RandomSubset(BaseArg&& base_arg, long k_arg, SeedArg&& seed=RandomSeed())
+      : base(std::forward<BaseArg>(base_arg))
+      , random_source(std::forward<SeedArg>(seed))
+      , k(k_arg)
    {
       if (POLYMAKE_DEBUG) {
          if (k<0 || k>get_base().size())
@@ -122,17 +133,16 @@ public:
       }
    }
 
-   RandomSubset(typename alias<ContainerRef>::arg_type base_arg, long k_arg, const SharedRandomState& s)
-      : base(base_arg), random_source(s), k(k_arg)
-   {
-      if (POLYMAKE_DEBUG) {
-         if (k<0 || k>get_base().size())
-            throw std::runtime_error("RandomSubset constructor - invalid size");
-      }
-   }
+   template <typename BaseArg,
+             typename=std::enable_if_t<std::is_constructible<alias_t, BaseArg>::value>>
+   RandomSubset(BaseArg&& base_arg, long k_arg)
+      : RandomSubset(std::forward<BaseArg>(base_arg), k_arg, RandomSeed()) {}
 
-   typedef RandomSubset_iterator<typename container_traits<ContainerRef>::iterator> iterator;
-   typedef RandomSubset_iterator<typename container_traits<ContainerRef>::const_iterator> const_iterator;
+   decltype(auto) get_base() { return *base; }
+   decltype(auto) get_base() const { return *base; }
+
+   using iterator = RandomSubset_iterator<typename container_traits<ContainerRef>::iterator>;
+   using const_iterator = RandomSubset_iterator<typename container_traits<ContainerRef>::const_iterator>;
 
    iterator begin()
    {
@@ -153,31 +163,23 @@ public:
 
    long size() const { return k; }
    bool empty() const { return k==0; }
-protected:
-   alias<ContainerRef> base;
-   UniformlyRandom<long> random_source;
-   long k;
-
-   typename alias<ContainerRef>::reference get_base() { return *base; }
-   typename alias<ContainerRef>::const_reference get_base() const { return *base; }
 };
 
 template <typename ContainerRef>
 struct spec_object_traits< RandomSubset<ContainerRef> >
    : spec_object_traits<is_container> {
-   static const bool is_temporary=true,
-                     is_always_const=effectively_const<ContainerRef>::value;
+   static constexpr bool is_temporary = true, is_always_const = is_effectively_const<ContainerRef>::value;
 };
 
 class RandomPermutation_iterator {
 public:
-   typedef forward_iterator_tag iterator_category;
-   typedef int value_type;
-   typedef const int& reference;
-   typedef const int* pointer;
-   typedef ptrdiff_t difference_type;
-   typedef RandomPermutation_iterator iterator;
-   typedef RandomPermutation_iterator const_iterator;
+   using iterator_category = forward_iterator_tag;
+   using value_type = int;
+   using reference = const int&;
+   using pointer = const int*;
+   using difference_type = ptrdiff_t;
+   using iterator = RandomPermutation_iterator;
+   using const_iterator = RandomPermutation_iterator;
 
    RandomPermutation_iterator()
       : rg(0) {}
@@ -192,6 +194,7 @@ public:
       : rg(0, random_arg) {}
 
    reference operator* () const { return perm_index.back(); }
+   pointer operator-> () const { return &operator*(); }
 
    iterator& operator++()
    {
@@ -227,12 +230,12 @@ template <>
 struct check_iterator_feature<RandomPermutation_iterator, end_sensitive> : std::true_type {};
 
 template <typename ContainerRef=sequence,
-          bool _direct=identical_minus_const_ref<ContainerRef, sequence>::value>
+          bool is_direct=same_pure_type<ContainerRef, sequence>::value>
 class RandomPermutation {
 public:
-   typedef int value_type;
-   typedef const int& reference;
-   typedef reference const_reference;
+   using value_type = int;
+   using reference = const int&;
+   using const_reference = reference;
 
    explicit RandomPermutation(const sequence& base_arg, const RandomSeed& seed=RandomSeed())
       : base(base_arg), random_source(seed) {}
@@ -246,8 +249,8 @@ public:
    RandomPermutation(int n, const SharedRandomState& random_arg)
       : base(0,n), random_source(random_arg) {}
 
-   typedef RandomPermutation_iterator iterator;
-   typedef iterator const_iterator;
+   using iterator = RandomPermutation_iterator;
+   using const_iterator = iterator;
 
    iterator begin() const
    {
@@ -268,23 +271,28 @@ protected:
 template <typename ContainerRef>
 class RandomPermutation<ContainerRef, false>
    : public indexed_subset_impl< RandomPermutation<ContainerRef, false>,
-                                 mlist< Container1Tag< ContainerRef >,
+                                 mlist< Container1RefTag< ContainerRef >,
                                         Container2Tag< RandomPermutation<> > > > {
-   typedef indexed_subset_impl<RandomPermutation> impl_t;
+   using impl_t = indexed_subset_impl<RandomPermutation>;
 protected:
-   alias<ContainerRef> base;
+   using alias_t = alias<ContainerRef>;
+   alias_t base;
    RandomPermutation<> perm;
 public:
-   explicit RandomPermutation(typename alias<ContainerRef>::arg_type base_arg, const RandomSeed& seed=RandomSeed())
-      : base(base_arg)
-      , perm(base->size(), seed) {}
+   template <typename BaseArg, typename SeedArg,
+             typename=std::enable_if_t<std::is_constructible<alias_t, BaseArg>::value &&
+                                       std::is_constructible<RandomPermutation<>, int, SeedArg>::value>>
+   RandomPermutation(BaseArg&& base_arg, SeedArg&& seed)
+      : base(std::forward<BaseArg>(base_arg))
+      , perm(base->size(), std::forward<SeedArg>(seed)) {}
 
-   RandomPermutation(typename alias<ContainerRef>::arg_type base_arg, const SharedRandomState& s)
-      : base(base_arg)
-      , perm(base->size(), s) {}
+   template <typename BaseArg,
+             typename=std::enable_if_t<std::is_constructible<alias_t, BaseArg>::value>>
+   explicit RandomPermutation(BaseArg&& base_arg)
+      : RandomPermutation(std::forward<BaseArg>(base_arg), RandomSeed()) {}
 
-   typename impl_t::container1& get_container1() { return *base; }
-   const typename impl_t::container1& get_container1() const { return *base; }
+   decltype(auto) get_container1() { return *base; }
+   decltype(auto) get_container1() const { return *base; }
    const RandomPermutation<>& get_container2() const { return perm; }
 };
 
@@ -294,74 +302,42 @@ struct spec_object_traits< RandomPermutation<ContainerRef> >
    static const bool is_temporary=true;
 };
 
-template <typename Container> inline
-RandomSubset<Container&>
-select_random_subset(Container& c, int k, const RandomSeed& seed=RandomSeed())
+template <typename Container>
+auto select_random_subset(Container&& c, int k, const RandomSeed& seed=RandomSeed())
 {
-   return RandomSubset<Container&>(c,k,seed);
+   return RandomSubset<Container>(std::forward<Container>(c), k, seed);
 }
 
-template <typename Container> inline
-RandomSubset<Container&>
-select_random_subset(Container& c, int k, const SharedRandomState& s)
+template <typename Container>
+auto select_random_subset(Container&& c, int k, const SharedRandomState& s)
 {
-   return RandomSubset<Container&>(c,k,s);
+   return RandomSubset<Container>(std::forward<Container>(c), k, s);
 }
 
-template <typename Container> inline
-RandomSubset<const Container&>
-select_random_subset(const Container& c, int k, const RandomSeed& seed=RandomSeed())
+template <typename Container>
+auto random_permutation(Container&& c, const RandomSeed& seed=RandomSeed())
 {
-   return RandomSubset<const Container&>(c,k,seed);
+   return RandomPermutation<Container>(std::forward<Container>(c), seed);
 }
 
-template <typename Container> inline
-RandomSubset<const Container&>
-select_random_subset(const Container& c, int k, const SharedRandomState& s)
+template <typename Container>
+auto random_permutation(Container&& c, const SharedRandomState& s)
 {
-   return RandomSubset<const Container&>(c,k,s);
-}
-
-template <typename Container> inline
-RandomPermutation<Container&>
-random_permutation(Container& c, const RandomSeed& seed=RandomSeed())
-{
-   return RandomPermutation<Container&>(c,seed);
-}
-
-template <typename Container> inline
-RandomPermutation<Container&>
-random_permutation(Container& c, const SharedRandomState& s)
-{
-   return RandomPermutation<Container&>(c,s);
-}
-
-template <typename Container> inline
-RandomPermutation<const Container&>
-random_permutation(const Container& c, const RandomSeed& seed=RandomSeed())
-{
-   return RandomPermutation<const Container&>(c,seed);
-}
-
-template <typename Container> inline
-RandomPermutation<const Container&>
-random_permutation(const Container& c, const SharedRandomState& s)
-{
-   return RandomPermutation<const Container&>(c,s);
+   return RandomPermutation<Container>(std::forward<Container>(c), s);
 }
 
 inline
 RandomPermutation<>
 random_permutation(int n, const RandomSeed& seed=RandomSeed())
 {
-   return RandomPermutation<>(n,seed);
+   return RandomPermutation<>(n, seed);
 }
 
 inline
 RandomPermutation<>
 random_permutation(int n, const SharedRandomState& s)
 {
-   return RandomPermutation<>(n,s);
+   return RandomPermutation<>(n, s);
 }
 
 } // end namespace pm
